"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeServer = exports.start = void 0;
const t = __importStar(require("runtypes"));
const restify_1 = __importDefault(require("restify"));
const botbuilder_runtime_1 = require("botbuilder-runtime");
const defaultOptions = {
    port: 3978,
};
/**
 * Start a bot using the runtime restify integration.
 *
 * @param applicationRoot application root directory
 * @param settingsDirectory settings directory
 * @param options options bag
 */
function start(applicationRoot, settingsDirectory, options = defaultOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const [services, configuration] = yield botbuilder_runtime_1.getRuntimeServices(applicationRoot, settingsDirectory);
        const server = yield makeServer(services, configuration);
        server.listen(options.port, () => {
            console.log(`server listening on port ${options.port}`);
        });
    });
}
exports.start = start;
/**
 * Create a server using the runtime restify integration.
 *
 * @param services runtime service collection
 * @param configuration runtime configuration
 * @returns a restify server ready to listen for connections
 */
function makeServer(services, configuration) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const { adapter, bot, customAdapters } = yield services.mustMakeInstances('adapter', 'bot', 'customAdapters');
        const server = restify_1.default.createServer();
        server.post('/api/messages', (req, res) => {
            adapter.processActivity(req, res, (turnContext) => __awaiter(this, void 0, void 0, function* () {
                yield bot.run(turnContext);
            }));
        });
        const adapters = (_a = (yield configuration.type(['runtimeSettings', 'adapters'], t.Array(t.Record({
            name: t.String,
            enabled: t.Union(t.Boolean, t.Undefined),
            route: t.String,
        }))))) !== null && _a !== void 0 ? _a : [];
        adapters
            .filter((settings) => settings.enabled)
            .forEach((settings) => {
            const adapter = customAdapters.get(settings.name);
            if (adapter) {
                server.post(`/api/${settings.route}`, (req, res) => {
                    adapter.processActivity(req, res, (turnContext) => __awaiter(this, void 0, void 0, function* () {
                        yield bot.run(turnContext);
                    }));
                });
            }
            else {
                console.warn(`Custom Adapter for \`${settings.name}\` not registered.`);
            }
        });
        server.on('upgrade', (req, socket, head) => __awaiter(this, void 0, void 0, function* () {
            const adapter = yield services.mustMakeInstance('adapter');
            adapter.useWebSocket(req, socket, head, (context) => __awaiter(this, void 0, void 0, function* () {
                yield bot.run(context);
            }));
        }));
        return server;
    });
}
exports.makeServer = makeServer;
//# sourceMappingURL=index.js.map